@@ -7,9 +7,13 @@ val reset : unit -> unit
 val file : Uutil.File.t -> unit
 val all : unit -> unit
 
+(* Check whether stop of all transfers has been requested. *)
+val isAll : unit -> bool
+val checkAll : unit -> unit (* Raises a transient exception *)
+
 (* Check whether an item is being aborted.  A transient exception is
    raised if this is the case. *)
 val check : Uutil.File.t -> unit
 
-(* Test whether the exeption is an abort exception. *)
+(* Test whether the exception is an abort exception. *)
 val testException : exn -> bool
