@@ -162,7 +162,12 @@ let catch_all f =
       (* Util.msg "Done catch_all...\n"; *)
     with Prefs.IllegalValue str -> raise (Util.Fatal str)
   with e ->
-    Util.msg "Unison server failed: %s\n" (Uicommon.exn2string e); exit 1;;
+    Util.msg "Unison server failed: %s\n" (Uicommon.exn2string e);
+    (* A final desperate attempt to print out some debug information.
+       If we are really-really out of memory then this may fail but
+       then it's unlikely we reach this point anyway. *)
+    if e = Out_of_memory then Gc.print_stat stderr;
+    exit 1
 
 let gui_safe_printf fmt =
   Printf.ksprintf (fun s ->
@@ -236,7 +241,18 @@ let init () = begin
               Uutil.myName)
     end;
     exit 0
-  with Not_found -> () end;
+  with
+  | Not_found -> ()
+  | Sys_error _ (* Broken pipe *) ->
+      (* A broken pipe (when stdout is piped to pager, for example) will cause
+         all output functions, including flush, to raise an exception. Catching
+         the exception here is not sufficient because stdout is implicitly
+         flushed on exit, which will again raise a broken pipe exception. The
+         only way to avoid [exit] raising a broken pipe exception is to close
+         [stdout] beforehand. *)
+      close_out_noerr stdout;
+      exit 0
+  end;
 
   (* Start a server if requested *)
   if Util.StringMap.mem serverPrefName argv then begin
