@@ -105,7 +105,7 @@ let deconstructRev path =
 let winAbspathRx = Rx.rx "([a-zA-Z]:)?(/|\\\\).*"
 let unixAbspathRx = Rx.rx "/.*"
 let is_absolute s =
-  if Util.osType=`Win32 then Rx.match_string winAbspathRx s
+  if Sys.win32 || Sys.cygwin then Rx.match_string winAbspathRx s
   else Rx.match_string unixAbspathRx s
 
 (* Function string2path: string -> path
@@ -151,7 +151,7 @@ let is_absolute s =
 *)
 let fromString str =
   let str0 = str in
-  let str = if Util.osType = `Win32 then Fileutil.backslashes2forwardslashes str else str in
+  let str = if Sys.win32 || Sys.cygwin then Fileutil.backslashes2forwardslashes str else str in
   if is_absolute str then
     raise (Util.Transient
              (Printf.sprintf "The path '%s' is not a relative path" str));
@@ -225,12 +225,8 @@ let followPred = Pred.create "follow"
       The syntax of \\ARG{pathspec} is \
       described in \\sectionref{pathspec}{Path Specification}.")
 
-let winHasReadlink =
-  Scanf.sscanf Sys.ocaml_version "%d.%d.%d" (fun x y z -> x > 4 || x = 4 && y >= 3)
-
 let followLink path =
-     (Util.osType = `Unix || Util.isCygwin || winHasReadlink)
-  && Pred.test followPred (toString path)
+  Pred.test followPred (toString path)
 
 let forceLocal p = p
 let makeGlobal p = p
