@@ -52,7 +52,7 @@ module StringMap : Map.S with type key = string
 val stringSetFromList : string list -> StringSet.t
 
 (* String manipulation *)
-val truncateString : string -> int -> string
+val truncateString : string -> int (* number of Unicode code points *) -> string
 val startswith : string -> string -> bool  (* STR,PREFIX *)
 val endswith : string -> string -> bool
 val findsubstring : ?reverse:bool -> string -> string -> int option
@@ -72,10 +72,6 @@ val padto : int -> string -> string
 (* ---------------------------------------------------------------------- *)
 (* Miscellaneous *)
 
-(* Architecture *)
-val osType : [`Unix | `Win32]
-val isCygwin: bool    (* osType will be `Win32 in this case *)
-
 (* Options *)
 val extractValueFromOption : 'a option -> 'a
 val option2string: ('a -> string) -> ('a option -> string)
